//VERTEX SHADER

// inputs
attribute mediump vec4 vertexAttribPosition;
attribute mediump vec3 vertexAttribNormal;
attribute mediump vec2 vertexAttribTexCoord0;

uniform highp mat4 modelScaleTransform;
uniform highp mat4 modelViewTransform;
uniform highp mat3 normalTransform;
uniform highp mat4 projectionViewTransform;

attribute vec2 vertexAttribInnerOutlineNormal;

uniform highp vec4 positionToPhoto;

uniform lowp float maxInnerOutline;

uniform highp float aspectAdjust;

uniform lowp float blurIntensity;
uniform lowp float additiveIntensity;
uniform lowp float contrastIntensity;
uniform lowp float brightnessIntensity;
uniform lowp float differenceIntensity;
uniform lowp float saturationIntensity;
uniform lowp float redblueIntensity;
uniform lowp float outlineIntensity;


uniform highp vec2 pixelSize;
uniform lowp float lodOffset;

uniform highp vec2 blendOffset;
uniform highp float blendRotate;
uniform highp float blendScale;

varying mediump vec2 v_redTextureCoordinate; // texture coordinates for red (if using 3d glasses)

// outputs (varyings for fragment shader)
varying mediump vec2 v_textureCoordinate;
varying mediump vec2 v_baseTextureCoordinate;

#ifdef USES_BLUR
varying lowp vec2 v_blurTexCoords[4];
varying lowp float bias;
#endif

varying lowp float localContrastIntensity;
varying lowp float localBrightnessIntensity;
varying lowp float localAdditiveIntensity;
varying lowp float localDifferenceIntensity;
varying lowp float localAlphaIntensity;
varying lowp float localSaturationIntensity;

lowp float zdepth;


highp vec2 rotateVec2(highp vec2 vector, highp float angle)
{
    highp float cs = cos(angle);
    highp float sn = sin(angle);

    highp vec2 outVector = vec2(vector.x * cs - vector.y * sn,
                                vector.x * sn + vector.y * cs);

    return outVector;
}

void setTextureCoordForPosition(highp vec4 position)
{
    zdepth = clamp(-position.z, 0.0, 1.0);

    // convert the gl_FragCoord into image space
    vec2 textureCoordinate2 = vec2((position.x - positionToPhoto.x) / positionToPhoto.z, (position.y - positionToPhoto.y) / positionToPhoto.w);

    v_baseTextureCoordinate = textureCoordinate2;

    highp float scale = 1.0+((blendScale-1.0) * zdepth);
    scale = 1.0/scale;

    // now tranlate the texture coord

    // translate to center on 0.5, 0.5
    textureCoordinate2 = textureCoordinate2 + vec2(-0.5, -0.5);




    // scale
    textureCoordinate2 = textureCoordinate2 * scale;

    // adjust aspect ratio for rotation
    textureCoordinate2.x = textureCoordinate2.x * aspectAdjust;

    // rotate
    textureCoordinate2 = rotateVec2(textureCoordinate2, blendRotate * zdepth);

    // offset
    textureCoordinate2 = textureCoordinate2 + (-blendOffset * zdepth);

    // unnadjust aspect ratio for rotation
    textureCoordinate2.x = textureCoordinate2.x / aspectAdjust;


    // translate to center correctly
    textureCoordinate2 = textureCoordinate2 + vec2(0.5, 0.5);

    v_textureCoordinate = textureCoordinate2;

#ifdef USES_REDBLUE
    // REDBLUE
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    lowp vec2 localRedBlueVector = vec2((zdepth - 0.5) * redblueIntensity  * 0.02, 0);

    localRedBlueVector = rotateVec2(localRedBlueVector, (blendRotate * zdepth)) * scale;

    v_redTextureCoordinate = v_textureCoordinate;
    v_redTextureCoordinate = v_redTextureCoordinate - localRedBlueVector;

    v_textureCoordinate = v_textureCoordinate + localRedBlueVector;

#endif

#ifdef USES_SEPARATE
    // REDBLUE
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    lowp vec2 localRedBlueVector = vec2((zdepth - 0.5) * redblueIntensity  * 0.02, 0);

    localRedBlueVector = rotateVec2(localRedBlueVector, (blendRotate * zdepth)) * scale;

    v_redTextureCoordinate = v_textureCoordinate;
    v_redTextureCoordinate = v_redTextureCoordinate - localRedBlueVector;

    v_textureCoordinate = v_textureCoordinate + localRedBlueVector;

#endif

}

void main()
{
    // POSITION AND LOCAL INTENSITY
    /////////////////////////////////////////////////////////////////

    // pass the texture coords on to the fragment shader as a varying attribute

    vec4 position = modelScaleTransform * vertexAttribPosition;

#ifdef USES_OUTLINE

#ifdef OUTLINE_NEGATIVE
    lowp float offset = clamp(abs(outlineIntensity), 0.0, 0.3); // clamp this so the outlines only get to a certain width

#else

    lowp float offset = clamp(outlineIntensity, 0.0, 1.0); // clamp this so the outlines only get to a certain width

#endif

    offset = clamp(offset * 0.03, 0.0, maxInnerOutline);

    lowp vec2 offsetVector = (offset * vertexAttribInnerOutlineNormal);
    offsetVector = offsetVector;

    position.xz = position.xz + offsetVector;

#endif

    position = modelViewTransform * position;

    // set the varying texture coord
    setTextureCoordForPosition(position);

    // set the position of the vertex (no transform, these are flat)
    gl_Position = projectionViewTransform * position;


#ifdef USES_BLUR

    // BLUR
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    lowp float localBlurIntensity = max((1.0 - zdepth) * blurIntensity, zdepth * -blurIntensity);

    // now create a texture coord array for the blur kernel
    //highp float scale = 1.0+((blendScale-1.0) * zdepth);

    highp float blurSize = localBlurIntensity * 0.035;

    bias = sqrt(max((blurSize * 1.23 / pixelSize.x)-1.0, 0.0)) - lodOffset;
    //bias = abs(bias);

    mediump float pixelYsize = (pixelSize.y / pixelSize.x);

    mediump vec2 blurVectorX = vec2(1.0, 0.3 * pixelYsize) * blurSize;
    mediump vec2 blurVectorY = vec2(-0.3, 1.0 * pixelYsize) * blurSize;


    // take 4 samples around the main texture coord

    // above and below
    v_blurTexCoords[ 0] = v_textureCoordinate - blurVectorY;
    v_blurTexCoords[ 1] = v_textureCoordinate - blurVectorX;
    v_blurTexCoords[ 2] = v_textureCoordinate + blurVectorY;
    v_blurTexCoords[ 3] = v_textureCoordinate + blurVectorX;

#endif


#ifdef USES_BRIGHTNESS
    // BRIGHTNESS
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    localBrightnessIntensity = max((1.0 - zdepth) * brightnessIntensity, zdepth * -brightnessIntensity);
#endif

#ifdef USES_CONTRAST
    // CONTRAST
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    localContrastIntensity = max((1.0 - zdepth) * contrastIntensity, zdepth * -contrastIntensity);
#endif

#ifdef USES_ADDITIVE
    // ADDITIVE
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    localAdditiveIntensity = max((1.0 - zdepth) * additiveIntensity, zdepth * -additiveIntensity);
#endif

#ifdef USES_DIFFERENCE
    // DIFFERENCE
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    localDifferenceIntensity = max((1.0 - zdepth) * differenceIntensity, zdepth * -differenceIntensity);
#endif

#ifdef USES_SATURATION
    // SATURATION
    /////////////////////////////////////////////////////////////////

    // if the intensity is below zero then flip the use of zdepth
    localSaturationIntensity = max((1.0 - zdepth) * saturationIntensity, zdepth * -saturationIntensity);
#endif



}

